@@ -1,48 +1,63 @@
 import json
 import logging
-from dataclasses import dataclass, field, asdict
+from dataclasses import field, asdict, is_dataclass
 from datetime import datetime
 from enum import StrEnum
 from pathlib import Path
 
-import microcore as mc
-from colorama import Fore, Style, Back
-from microcore.utils import file_link
 import textwrap
+import microcore as mc
+from microcore.utils import file_link
+from colorama import Fore, Style, Back
+from pydantic.dataclasses import dataclass
 
-from .constants import JSON_REPORT_FILE_NAME, HTML_TEXT_ICON
+from .constants import JSON_REPORT_FILE_NAME, HTML_TEXT_ICON, HTML_CR_COMMENT_MARKER
 from .project_config import ProjectConfig
-from .utils import syntax_hint, block_wrap_lr, max_line_len
+from .utils import syntax_hint, block_wrap_lr, max_line_len, remove_html_comments, filter_kwargs
 
 
 @dataclass
-class Issue:
+class RawIssue:
     @dataclass
     class AffectedCode:
         start_line: int = field()
         end_line: int | None = field(default=None)
+        proposal: str | None = field(default="")
+
+    title: str = field()
+    details: str | None = field(default="")
+    severity: int | None = field(default=None)
+    confidence: int | None = field(default=None)
+    tags: list[str] = field(default_factory=list)
+    affected_lines: list[AffectedCode] = field(default_factory=list)
+
+
+@dataclass
+class Issue(RawIssue):
+    @dataclass
+    class AffectedCode(RawIssue.AffectedCode):
         file: str = field(default="")
-        proposal: str = field(default="")
         affected_code: str = field(default="")
 
         @property
         def syntax_hint(self) -> str:
             return syntax_hint(self.file)
 
-    id: str = field()
-    title: str = field()
-    details: str = field(default="")
-    severity: int | None = field(default=None)
-    confidence: int | None = field(default=None)
-    tags: list[str] = field(default_factory=list)
+    id: int | str = field(kw_only=True)
     file: str = field(default="")
     affected_lines: list[AffectedCode] = field(default_factory=list)
 
-    def __post_init__(self):
-        self.affected_lines = [
-            Issue.AffectedCode(**dict(file=self.file) | i)
-            for i in self.affected_lines
-        ]
+    @staticmethod
+    def from_raw_issue(file: str, raw_issue: RawIssue | dict, issue_id: int | str) -> "Issue":
+        if is_dataclass(raw_issue):
+            raw_issue = asdict(raw_issue)
+        params = filter_kwargs(Issue, raw_issue | {"file": file, "id": issue_id})
+        for i, obj in enumerate(params["affected_lines"]):
+            d = obj if isinstance(obj, dict) else asdict(obj)
+            params["affected_lines"][i] = Issue.AffectedCode(
+                **filter_kwargs(Issue.AffectedCode, {"file": file} | d)
+            )
+        return Issue(**params)
 
     def github_code_link(self, github_env: dict) -> str:
         url = (
@@ -79,19 +94,20 @@ class Report:
             for issue in issues
         ]
 
+    def register_issues(self, issues: dict[str, list[RawIssue | dict]]):
+        for file, file_issues in issues.items():
+            for issue in file_issues:
+                self.register_issue(file, issue)
+
+    def register_issue(self, file: str, issue: RawIssue | dict):
+        if file not in self.issues:
+            self.issues[file] = []
+        total = len(self.plain_issues)
+        self.issues[file].append(Issue.from_raw_issue(file, issue, issue_id=total + 1))
+        self.total_issues = total + 1
+
     def __post_init__(self):
-        issue_id: int = 0
-        for file in self.issues.keys():
-            self.issues[file] = [
-                Issue(
-                    **{
-                        "id": (issue_id := issue_id + 1),
-                        "file": file,
-                    } | issue
-                )
-                for issue in self.issues[file]
-            ]
-        self.total_issues = issue_id
+        self.total_issues = len(self.plain_issues)
 
     def save(self, file_name: str = ""):
         file_name = file_name or JSON_REPORT_FILE_NAME
@@ -125,6 +141,8 @@ class Report:
             block_wrap_lr=block_wrap_lr,
             max_line_len=max_line_len,
             HTML_TEXT_ICON=HTML_TEXT_ICON,
+            HTML_CR_COMMENT_MARKER=HTML_CR_COMMENT_MARKER,
+            remove_html_comments=remove_html_comments,
             **config.prompt_vars
         )
 
